@@ -12,5 +12,5 @@ SERVICE_NOT_REGISTERED	= 204
 SERVICE_NO_SUCH_NOUN	= 205
 SERVICE_INTERNAL_KEY	= 206
 SERVICE_CRASHED			= 207
-SERVICE_NO_BODY			= 208
+SERVICE_NO_DATA			= 208
 SERVICE_NO_SESSION		= 209
\ No newline at end of file
